#include "damsel.h"
#include "damsel-hdf5.h"

/*! fileside storage of entity definitions. this mirrors the parameters passed
    to DMSLentity_define. this is used differently to the infile
    tag_container and handle_container structs - those are described to
    HDF5, but this struct is described to HDF5 as a byte array - further
    damsel type work will have to happen before this is described further
    to the damsel and lower storage layers.

    this is a struct that needs to be mapped out to the concrete file using
    the type system - for now its described as an opaque type and the fileside
    should store it so, but a richer type system should allow richer storage.

    these are a bit like application-layer structs, but for the connectivity
    code.
*/
typedef struct {
  int entity_type;
  int vertices_per_entity;
  damsel_handle entity_container_handle; // this is a file-side handle
  damsel_handle vertex_container_handle; // this is a file-side handle
} damsel_infile_entity_definition;

typedef struct {
  int entity_type;
  int vertices_per_entity;
  damsel_container entity_container_id;  // vertex container in app side
  damsel_container vertex_container_id;  // vertex container in app side
} damsel_internal_entity_definition;

int DMSLentity_get_count(damsel_model m);

damsel_err_t DMSLentity_get_entity_infos(damsel_model m, damsel_entity_buf_type *buf);
damsel_err_t DMSLentity_close_entity_infos(damsel_model m, damsel_entity_buf_type *buf);

damsel_err_t DMSLmodel_close_entity_infos(damsel_model m, damsel_entity_buf_type *buf);

void entities_initialize_for_read(damsel_attached_file *af);
