/* damsel provenance dumper.

This interacts with the HDF5 datasets directly, rather than
using the damsel model interface, because it is metadata about
the model, rather than information in the model.

*/

#include <assert.h>
#include <hdf5.h>
#include <stdio.h>

#include "damsel.h"
#include "damsel-internal.h"
#include "damsel-versioning.h"

int main(int argc, char **argv) {
  int ret;
  fprintf(stderr, "damsel provenance dumper\n");
  assert(argc==2);

  char *filename = argv[1];

  hid_t file_id = H5Fopen(filename, H5F_ACC_RDWR, H5P_DEFAULT);
  assert(file_id >= 0);

  hid_t v_dataset = H5Dopen(file_id, DAMSEL_DATASET_VERSIONING, H5P_DEFAULT);
  assert(v_dataset >= 0);

  hid_t v_space = H5Dget_space(v_dataset);
  assert(v_space >= 0);

  hsize_t v_count = H5Sget_simple_extent_npoints(v_space);

  fprintf(stderr, "There are %d versions in this file.\n", (int) v_count);

  damsel_hdf5_infile_versioning_watermark *wms = malloc(sizeof(damsel_hdf5_infile_versioning_watermark) * v_count);
  assert(wms != NULL);

  hid_t v_type = create_versioning_type();

  ret = H5Dread(v_dataset, v_type, v_space, v_space, H5P_DEFAULT, wms);
  assert(ret >= 0);

  printf("{\n");

  printf("\"entity\": {\n");
// represent the file as an entity
  printf("  \"f:file\":{\n");

  printf("    \"benc\":{\n");
  printf("      \"relative-uri\":\"%s\"\n", filename);
  printf("    }\n");
  printf("}");
  int i;
  for(i=0; i<v_count; i++) {

// now represent each version of the file
// as an entity
    printf(",\n");
    printf("  \"f:v%d\":{\n", i);
    printf("  }\n");
  }
  printf("},\n");

  printf(" \"specializationOf\":{\n");
  for(i=0; i<v_count; i++) {

// and establish a link between this version and
// the parent file
    if(i>0) printf(",");
    printf("  \"_:spec%d\":{\n", i);
    printf("    \"prov:specificEntity\":\"f:v%d\",\n", i);
    printf("    \"prov:generalEntity\":\"f:file\"\n");
    printf("  }\n");
  }
  printf("},\n");
  printf("\"wasDerivedFrom\":{\n");
  for(i=0; i<v_count; i++) {
// and establish a link between this version and
// the previous version (if any)
    if(i>0) {
      if(i>1) printf(",");
      printf("  \"_:deriv%d\":{\n", i);
      printf("    \"prov:generatedEntity\":\"f:v%d\",\n", i);
      printf("    \"prov:usedEntity\":\"f:v%d\"\n", i-1);
      printf("  }\n");
    }
  }
  printf("}\n");

  printf(",\n");

  printf("  \"activity\":{\n");

  for(i=0; i<v_count; i++) {
    if(i>0) printf(",");
    printf("    \"f:act%d\":{\n", i);

    printf("    \"damsel:transfer-end-timestamp\":%ld,\n", wms[i].timestamp);
    printf("    \"damsel:mpi-comm-world-size\":%ld\n", wms[i].comm_world_size);

    printf("    }\n");
  }

  printf("  },\n");

  printf("  \"wasGeneratedBy\":{\n");
  for(i=0;  i<v_count; i++) {
    if(i>0) printf(",");
    printf("  \"_:genr%d\":{\n", i);
    printf("  \"prov:entity\":\"f:v%d\",\n", i);
    printf("  \"prov:activity\":\"f:act%d\",\n", i);
    printf("  \"prov:time\":%ld\n", wms[i].timestamp); // TODO: should be w3c format not a unix seconds count
    printf("  }\n");
  }
  printf("}\n");

  printf("}\n");

  ret = H5Sclose(v_space);
  assert(ret >= 0);

  ret = H5Dclose(v_dataset);
  assert(ret >= 0);


  ret = H5Fclose(file_id);
  assert(ret >= 0);

  return 0;
}

