#include "damsel-hdf5.h"

/* data structures for damsel file versioning */

/* each row of the hdf5 version table dataset will
   record a high watermark for every damsel internal
   table.
*/

#define MAX_URI_SIZE 256

typedef struct {

// a version will store high water marks (the
// current row count) for each of the below
// containers, when a transfer is made.

  long collections;
  long entity_connectivity;
  long seq_handle_containers;
  long tag_containers;
  long vector_handle_containers;
  long timestamp; // unix time_t value
  long comm_world_size; // size of COMM_WORLD, not of the communicator used for damsel writes

  // note that damsel-handle-allocator is ommitted
  // benc thinks, at the moment, that it is not necessary for versioning

} damsel_hdf5_infile_versioning_watermark;

typedef struct {
  long version; // the row number in the versions table
  char uri[MAX_URI_SIZE]; // the URI that this row depends on
} damsel_hdf5_infile_versioning_dependency;

void initialize_versioning_type(damsel_attached_file *file);
hid_t get_versioning_type(damsel_attached_file *file);
hid_t create_versioning_type();

void store_version(damsel_model_internal *src_s, damsel_attached_file *fs);

void store_dependencies(damsel_model_internal *src_s, damsel_attached_file *fs);

/** allow an input entity to be declared to damsel so that it is recorded
    in the provenance record. The parameter should be a URI; this can be
    a filename relative to the location of the damsel output file, for
    example (though that will break when the damsel output file is moved,
    but hey ho)
*/
void damsel_provenance_declare_input(char *uri);

