#ifndef __H5MULTIDATASET_H
#define __H5MULTIDATASET_H

#ifdef DAMSEL_HDF5
#ifndef HAVE_H5DWRITE_MULTI

#ifdef __cplusplus
extern "C" {
#endif

#include <hdf5.h>
#include <stdlib.h>

#define H5D_MULTI_CLOSE_DATASET  (0x0001u)  /* close dataset    */
#define H5D_MULTI_CLOSE_MSPACE   (0x0002u)  /* close mem space  */
#define H5D_MULTI_CLOSE_FSPACE   (0x0004u)  /* close file space */
#define H5D_MULTI_CLOSE_DATATYPE (0x0008u)  /* close datatype   */
#define H5D_MULTI_FREE_BUF       (0x00010u) /* free data buffer */
#define H5D_MULTI_CLOSE_ALL      (0x000fu)

/* can be casted from H5D_rw_multi_t and H5D_rw_multi_t */
typedef struct H5D_rw_multi_t
{
    hid_t dset_id;		/* as per H5Dread/write/close() */
    hid_t file_space_id;	/* as per H5Dread/write/close() */
    void * rbuf;             /* as per H5Dread/write/close() */
    const void * wbuf;             /* as per H5Dread/write/close() */
    hid_t mem_type_id; 		/* as per H5Dread/write/close() */
    hid_t mem_space_id; 	/* as per H5Dread/write/close() */
} H5D_rw_multi_t;

#if 0
/**
This function performs collective closes multiple datasets.
    
@param  count         -- the number of datasets.
@param  info     -- the array of dataset information.
					  
@return a non-negative value if successful; otherwise returns a negative value..
*/
herr_t H5Dclose_multi(size_t count, H5D_rw_multi_t info[]);
#endif
					
/**
This function performs collective reads from multiple datasets.
    
@param  count         -- the number of datasets.
@param  info     -- the array of dataset information and read buffers.
@param  xfer_plist_id -- dataset access property.
					  
@return a non-negative value if successful; otherwise returns a negative value..
*/
herr_t H5Dread_multi(hid_t fid, 
	size_t count,
                    H5D_rw_multi_t info[],
                    hid_t xfer_plist_id);
				

/**
This function performs collective writed to multiple datasets.
    
@param  count         -- the number of datasets.
@param  info     -- the array of dataset information and write buffers.
@param  xfer_plist_id -- dataset access property.
					  
@return a non-negative value if successful; otherwise returns a negative value.
*/								  
herr_t H5Dwrite_multi(hid_t fid, size_t count,
                    H5D_rw_multi_t info[],
                    hid_t xfer_plist_id);
					
#ifdef __cpluspluc
}
#endif

#endif 
#endif	
#endif

				
