#include <stdio.h>
#include <assert.h>
#include <unistd.h>

#include "damsel.h"
#define FILENAME "output/test-connectivity-fast.h5"

/* this test will define some entities and their connectivity and write it
   out to a file.
   It is based on doc/DamselCoreAPIExample.txt in svn r66 */

int main(int argc, char **argv) {
  printf("%s\n", argv[0]);

#ifndef DAMSEL_SINGLETHREAD
  int res;
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &res);
  assert(res == MPI_THREAD_MULTIPLE);
#else
  MPI_Init(&argc, &argv);
#endif

  damsel_library lib;
  damsel_err_t err = DMSLlib_init(&lib);

  damsel_model model;
  err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
  unlink(FILENAME);
  damsel_trait_bundle tb;
  err = DMSLtrait_create(&tb);
  DMSLtrait_put(tb, "mode", "WRITE");

  DMSLmodel_attach(model, FILENAME, MPI_COMM_WORLD, tb);
  DMSLtrait_release(tb);

  damsel_handle vertex_start = 10;

  /* this will define a sequence of 25 handles, numbered 10, 11, ... */
  damsel_container vertex_container_id;
  err = DMSLcontainer_create_sequence(model,
    /* first handle = */ vertex_start, /* count = */ 6, 
    /* increment = */ 1, &vertex_container_id);

  DMSLmodel_map_handles_inventing_file_handles(vertex_container_id);

  damsel_handle edge_start = 100;
  damsel_container edge_container_id;
  err = DMSLcontainer_create_sequence(model,
    /* first handle = */ edge_start, /* count = */ 5, 
    /* increment = */ 2, &edge_container_id);

  DMSLmodel_map_handles_inventing_file_handles(edge_container_id);

  damsel_handle edge_connect[10] = {10, 11, 11, 12, 12, 13, 13, 14, 14, 15};
  DMSLentity_define_fast(edge_container_id, /* entity type = */ DAMSEL_ENTITY_TYPE_EDGE,
    /* # of vertices per entity = */ 2, edge_connect);

  printf("test-connectivity-fast: container_hc = %p, vertex_container_hc = %p\n", (void *)edge_container_id, (void *)vertex_container_id);

  DMSLmodel_transfer_sync(model, DAMSEL_TRANSFER_TYPE_WRITE);
  DMSLmodel_close(model);
  DMSLcontainer_close(vertex_container_id);
  DMSLcontainer_close(edge_container_id);

  DMSLlib_finalize(lib);

  MPI_Finalize();
  printf("done!\n");
  return 0;
}

