/* test creating a simple FLASH grid and assigning different variables to the cells */
/* maybe in this version I will not be able to capture a whole lot of the structural and 
 * connectivity information but at least I can create block handles and tag them with 
 * solution data and tree metadata ^.^ */

#include "test-flash.h"

#define MAX_BLOCKS 17

#define FILENAME "output/test-flash.h5"

int main(int argc, char **argv) {
  
 // int i, j;
/*  for(i=0;i<MAX_BLOCKS;i++){
	for(j=0;j<24;j++)
		unk[i][j]=i+j; //solution data array, considering one variable and 2D data here
  }
*/
#ifndef DAMSEL_SINGLETHREAD
  int res;
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &res);
  assert(res == MPI_THREAD_MULTIPLE);
#else
  MPI_Init(&argc, &argv);
#endif

  damsel_library lib;
  damsel_err_t err = DMSLlib_init(&lib);

  damsel_model model;
  err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
  assert(model != NULL);
  damsel_model_internal *ms = (damsel_model_internal *)model;
  assert(ms != NULL);

  DMSLmodel_attach(model, argv[1], MPI_COMM_WORLD, NULL);
  
  /*creating array data type for unk, and other block metadata tags*/

  size_t unk_array_dim[] = {24};
  damsel_data_type unk_array_type = DMSLtype_array_create(DAMSEL_DATA_TYPE_DOUBLE, 1, unk_array_dim);
/*
  size_t coords_array_dim[] = {2};
  damsel_data_type coords_array_type = DMSLtype_array_create(DAMSEL_DATA_TYPE_DOUBLE, 1, coords_array_dim);

  size_t bndbox_array_dim[] = {4};
  damsel_data_type bndbox_array_type = DMSLtype_array_create(DAMSEL_DATA_TYPE_DOUBLE, 1, bndbox_array_dim);
*/
  size_t gid_array_dim[] = {15};
  damsel_data_type gid_array_type = DMSLtype_array_create(DAMSEL_DATA_TYPE_INTEGER, 1, gid_array_dim);

  /*end of array type definition*/

  /*Define tags to store the metadata and data -- (this handle doesn't need to match up with the handle used to write data)*/
  damsel_handle block_collection_handle = 1000; //I'm calling it block collection handle to be consistent with FLASH terminology and I am creating a sequence of block handles of e.g. 17 blocks example case. 

  /* we'll create a sequence handle collection with 17 handles in it. Each handle will correspond to a flash block*/

  damsel_container block_collection_id;
  err = DMSLcontainer_create_sequence(model, 1, 17, 1, &block_collection_id); //base_handle is 1, and step size is 1 - just being consistent with FLASH

  damsel_handle unk_tag_handle = 10000; 
  DMSLtag_define(model, &unk_tag_handle, unk_array_type, "UNK"); //corresponding to the unknown array (4D/5D) from FLASH for solution data
/*
  damsel_handle lrefine_tag_handle = 10001;
  DMSLtag_define(model, &lrefine_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "lrefine");

  damsel_handle whichchild_tag_handle = 10002;
  DMSLtag_define(model, &whichchild_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "whichchild");

  damsel_handle nodetype_tag_handle = 10003;
  DMSLtag_define(model, &nodetype_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "nodetype");

  damsel_handle coord_tag_handle = 10004;
  DMSLtag_define(model, &coord_tag_handle, coords_array_type, "coordinates");

  damsel_handle size_tag_handle = 10005;
  DMSLtag_define(model, &size_tag_handle, coords_array_type, "size");

  damsel_handle bndbox_tag_handle = 10006;
  DMSLtag_define(model, &bndbox_tag_handle, bndbox_array_type, "boundingbox");
*/
  /*Question: so the type of these tags is damsel handles ?? can the value/data for a tag is of type handle?
 * right now I'm using type integer bcz handles are stored as integers eventually*/

  damsel_handle gid_tag_handle = 10007;
  DMSLtag_define(model, &gid_tag_handle, gid_array_type, "parent_children_neighbor");

  damsel_handle tag_collection_handle = 0x50022;
  damsel_container tag_collection_id;
  err = DMSLcontainer_create_sequence(model, unk_tag_handle, 1, 1, &tag_collection_id);

  int tag_infos_count;
  int found = 0;
  assert(ms->attached_file != NULL);
  DMSLmodel_get_tuple_count(ms, NULL, &tag_infos_count);
 
  size_t amt = tag_infos_count * sizeof(damsel_tag_buf_type);
  //printf("Will allocate %zd bytes for tag buffers. sizeof buf =%zd. tag_infos_count = %zd\n", amt, sizeof(damsel_tag_buf_type), (size_t) tag_infos_count);
  
  damsel_tag_buf_type *tag_buf = malloc(amt);
  assert(tag_buf != NULL);

  printf("getting tag infos\n");
  DMSLmodel_get_tag_infos(model, tag_buf);
  printf("got tag infos\n");

  int tag_ix;
  for(tag_ix = 0; tag_ix < tag_infos_count; tag_ix++){
     printf("checking tag %d with name: %s\n", tag_ix, tag_buf[tag_ix].name);
     damsel_container file_tag_collection_id;
     err = DMSLcontainer_create_sequence((damsel_model)ms->attached_file, *tag_buf[tag_ix].tag_handle, 1, 1, &file_tag_collection_id);
     DMSLmodel_map_handles(tag_collection_id, file_tag_collection_id);

/*     if(strcmp(tag_buf[tag_ix].name, "lrefine"))
	{
	printf("lrefine matched\n");
	assert(found == 0);
	printf("tag handle in file is 0x%lx\n", *tag_buf[tag_ix].tag_handle);
	damsel_container file_tag_collection_id;
        err = DMSLcontainer_create_sequence((damsel_model)ms->attached_file, *tag_buf[tag_ix].tag_handle, 1, 1, &file_tag_collection_id);
	DMSLmodel_map_handles(tag_collection_id, file_tag_collection_id);
	found = 1;
     }else{
	printf("skipping");
     }*/
  } 

  //assert(found == 1);
  DMSLmodel_map_tag(unk, block_collection_id, &unk_tag_handle);
  damsel_handle filestore_block_collection = 0x910000;

  //damsel_container filestore_block_collection_id;
  //err = DMSLcontainer_create_sequence_with_handle((damsel_store)ms->attached_file_store, filestore_block_collection, 17, 1, 1, &filestore_block_collection_id);
  damsel_container filestore_block_collection_id;
  err = DMSLcontainer_create_sequence((damsel_model)ms->attached_file, 1, 17, 1, &filestore_block_collection_id);
  DMSLmodel_map_handles(block_collection_id, filestore_block_collection_id);

/* now set the values of the somedata tag on the handles */
/*  DMSLmodel_map_tag(unk, block_collection_id, &unk_tag_handle);
  DMSLmodel_map_tag(lrefine, block_collection_id, &lrefine_tag_handle);
  DMSLmodel_map_tag(which_child, block_collection_id, &whichchild_tag_handle);
  DMSLmodel_map_tag(nodetype, block_collection_id, &nodetype_tag_handle);
  DMSLmodel_map_tag(bnd_box, block_collection_id, &bndbox_tag_handle);
  DMSLmodel_map_tag(block_coords, block_collection_id, &coord_tag_handle);
  DMSLmodel_map_tag(block_size, block_collection_id, &size_tag_handle);
  DMSLmodel_map_tag(gid, block_collection_id, &gid_tag_handle);
*/

  /* at this point, we have all the data in memory and we have defined corresponding handles and tags. 
 * Now the next step is to invent file handles - I'm not very clear about them yet but will give them a try. 
  */
/*
  unlink(FILENAME); // maybe it doesn't exist, so we'll just silently ignore any error
  DMSLmodel_attach(model, FILENAME, MPI_COMM_WORLD, NULL);

  DMSLmodel_map_handles_inventing_file_handles(block_collection_id);

  DMSLmodel_map_single_handle_inventing_file_handle(model, unk_tag_handle); 
  DMSLmodel_map_single_handle_inventing_file_handle(model, lrefine_tag_handle);
  DMSLmodel_map_single_handle_inventing_file_handle(model, whichchild_tag_handle);
  DMSLmodel_map_single_handle_inventing_file_handle(model, nodetype_tag_handle);
  DMSLmodel_map_single_handle_inventing_file_handle(model, coord_tag_handle);
  DMSLmodel_map_single_handle_inventing_file_handle(model, size_tag_handle);
  DMSLmodel_map_single_handle_inventing_file_handle(model, bndbox_tag_handle);
*//* Now we can tell damsel to execute everything. */

  damsel_request_t req;
  DMSLmodel_transfer_async(model, DAMSEL_TRANSFER_TYPE_READ, &req);
  damsel_status_t status;
  DMSLmodel_wait(req, &status);

  DMSLmodel_close(model);

  DMSLlib_finalize(lib);

  MPI_Finalize();
  return 0;
}

