/* test creating a simple FLASH grid and assigning different variables to the cells */

#include "test-flash.h"

#include <stdio.h>

#define MAX_BLOCKS 17
#define FILENAME "output/test-flash.h5"

int main(int argc, char **argv) {
  
  int i, j;
  for(i=0;i<MAX_BLOCKS;i++){
	for(j=0;j<24;j++)
		unk[i][j]=i+j; //solution data array, considering one variable and 2D data here
  }

#ifndef DAMSEL_SINGLETHREAD
  int res;
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &res);
  assert(res == MPI_THREAD_MULTIPLE);
#else
  MPI_Init(&argc, &argv);
#endif

  damsel_library lib;
  damsel_err_t err = DMSLlib_init(&lib);

  /*creating array data type for unk, and other block metadata tags*/

  size_t unk_array_dim[] = {24};
  damsel_data_type unk_array_type = DMSLtype_array_create(DAMSEL_DATA_TYPE_DOUBLE, 1, unk_array_dim);

  size_t coords_array_dim[] = {2};
  damsel_data_type coords_array_type = DMSLtype_array_create(DAMSEL_DATA_TYPE_DOUBLE, 1, coords_array_dim);

  size_t bndbox_array_dim[] = {4};
  damsel_data_type bndbox_array_type = DMSLtype_array_create(DAMSEL_DATA_TYPE_DOUBLE, 1, bndbox_array_dim);

  size_t gid_array_dim[] = {15};
  damsel_data_type gid_array_type = DMSLtype_array_create(DAMSEL_DATA_TYPE_INTEGER, 1, gid_array_dim);

  /*end of array type definition*/

  damsel_model model;
  err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
  assert(model != NULL);
  damsel_model_internal *ms = (damsel_model_internal *)model;
  assert(ms != NULL);

  // damsel_handle block_collection_handle = 1000; //I'm calling it block collection handle to be consistent with FLASH terminology and I am creating a sequence of block handles of e.g. 17 blocks example case. 

  /* we'll create a sequence handle collection with 17 handles in it. Each handle will correspond to a flash block*/
/*
  damsel_container block_collection_id;
  err = DMSLcontainer_create_sequence(model, 1, 17, 1, &block_collection_id); //base_handle is 1, and step size is 1 - just being consistent with FLASH
*/
  damsel_handle block_id [17]={1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17};
  damsel_container block_collection_id;
  err = DMSLcontainer_create_vector(model, block_id, 17, &block_collection_id);
  /*we'll create a sequence of edge handles to connect the blocks that are at the same level, each block has got two neighbors in our 17 block example case, and block 1 has no neighbors*/
  damsel_container block_connect_collection_id;
  err = DMSLcontainer_create_sequence(model, 100, 32, 1, &block_connect_collection_id);

  damsel_handle block_connect[32] = {2, 11, 2, 12, 12, 13, 11, 13, 3, 9, 3, 8, 8, 9, 8, 10, 14, 15, 14, 16, 16, 17, 15, 17, 4, 5, 4, 6, 5, 7, 6, 7};
  damsel_container block_connect_handle_collection_id;
  err = DMSLcontainer_create_vector(model, block_connect, 32, &block_connect_handle_collection_id);

  /*We can define edges between the blocks sharing a boundary/face */

  DMSLentity_define(block_connect_collection_id, DAMSEL_ENTITY_TYPE_EDGE, 2, block_connect_handle_collection_id);

  /*defining tage handles for solution data and other structural information*/
  damsel_handle unk_tag_handle = 10000; 
  DMSLtag_define(model, &unk_tag_handle, unk_array_type, "UNK"); //corresponding to the unknown array (4D/5D) from FLASH for solution data

  damsel_handle lrefine_tag_handle = 10001;
  DMSLtag_define(model, &lrefine_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "lrefine");

  damsel_handle whichchild_tag_handle = 10002;
  DMSLtag_define(model, &whichchild_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "whichchild");

  damsel_handle nodetype_tag_handle = 10003;
  DMSLtag_define(model, &nodetype_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "nodetype");

  damsel_handle coord_tag_handle = 10004;
  DMSLtag_define(model, &coord_tag_handle, coords_array_type, "coordinates");

  damsel_handle size_tag_handle = 10005;
  DMSLtag_define(model, &size_tag_handle, coords_array_type, "size");

  damsel_handle bndbox_tag_handle = 10006;
  DMSLtag_define(model, &bndbox_tag_handle, bndbox_array_type, "boundingbox");

  /*Question: so the type of these tags is damsel handles ?? can the value/data for a tag is of type handle?
 * right now I'm using type integer bcz handles are stored as integers eventually*/

  damsel_handle gid_tag_handle = 10007;
  DMSLtag_define(model, &gid_tag_handle, gid_array_type, "parent_children_neighbor");

  damsel_handle parent1_tag_handle = 10008;
  damsel_handle child_handles1[4] = {block_id[1], block_id[10], block_id[11], block_id[12]};
  damsel_container child_collection_id_1;
  err = DMSLcontainer_create_vector(model, child_handles1, 4, &child_collection_id_1);
  DMSLtag_define(model, &parent1_tag_handle, DAMSEL_DATA_TYPE_HANDLE, "parent1");

  damsel_handle parent2_tag_handle = 10009;
  damsel_handle child_handles2[4] = {block_id[2], block_id[7], block_id[8], block_id[9]};
  damsel_container child_collection_id_2;
  err = DMSLcontainer_create_vector(model, child_handles2, 4, &child_collection_id_2);
  DMSLtag_define(model, &parent2_tag_handle, DAMSEL_DATA_TYPE_HANDLE, "parent2");

  damsel_handle parent3_tag_handle = 10010;
  damsel_handle child_handles3[4] = {block_id[3], block_id[4], block_id[5], block_id[6]};
  damsel_container child_collection_id_3;
  err = DMSLcontainer_create_vector(model, child_handles3, 4, &child_collection_id_3);
  DMSLtag_define(model, &parent3_tag_handle, DAMSEL_DATA_TYPE_HANDLE, "parent3");

  damsel_handle parent4_tag_handle = 10011;
  damsel_handle child_handles4[4] = {block_id[13], block_id[14], block_id[15], block_id[16]};
  damsel_container child_collection_id_4;
  err = DMSLcontainer_create_vector(model, child_handles4, 4, &child_collection_id_4);
  DMSLtag_define(model, &parent4_tag_handle, DAMSEL_DATA_TYPE_HANDLE, "parent4");

/*Saba: can we tag same sequence of vertex handles with multiple tags/tag handles??*/ 
  /* now set the values of the somedata tag on the handles */
  DMSLmodel_map_tag(unk, block_collection_id, &unk_tag_handle);
  DMSLmodel_map_tag(lrefine, block_collection_id, &lrefine_tag_handle);
  DMSLmodel_map_tag(which_child, block_collection_id, &whichchild_tag_handle);
  DMSLmodel_map_tag(nodetype, block_collection_id, &nodetype_tag_handle);
  DMSLmodel_map_tag(bnd_box, block_collection_id, &bndbox_tag_handle);
  DMSLmodel_map_tag(block_coords, block_collection_id, &coord_tag_handle);
  DMSLmodel_map_tag(block_size, block_collection_id, &size_tag_handle);
  DMSLmodel_map_tag(gid, block_collection_id, &gid_tag_handle);

  DMSLmodel_map_tag(&block_id[0],child_collection_id_1, &parent1_tag_handle); 
  DMSLmodel_map_tag(&block_id[1],child_collection_id_2, &parent2_tag_handle);
  DMSLmodel_map_tag(&block_id[2],child_collection_id_3, &parent3_tag_handle);
  DMSLmodel_map_tag(&block_id[12],child_collection_id_4, &parent4_tag_handle);
  
 /* at this point, we have all the data in memory and we have defined corresponding handles and tags. 
  * Now the next step is to invent file handles - I'm not very clear about them yet but will give them a try. 
  */

  unlink(FILENAME); // maybe it doesn't exist, so we'll just silently ignore any error
  damsel_trait_bundle tb;
  err = DMSLtrait_create(&tb);
  DMSLtrait_put(tb, "mode", "WRITE");

  DMSLmodel_attach(model, FILENAME, MPI_COMM_WORLD, tb);
  DMSLtrait_release(tb);

  DMSLmodel_map_handles_inventing_file_handles(block_collection_id);
  DMSLmodel_map_handles_inventing_file_handles(block_connect_collection_id);

  DMSLmodel_map_single_handle_inventing_file_handle(model, gid_tag_handle);
  DMSLmodel_map_single_handle_inventing_file_handle(model, unk_tag_handle); 
  DMSLmodel_map_single_handle_inventing_file_handle(model, lrefine_tag_handle);
  DMSLmodel_map_single_handle_inventing_file_handle(model, whichchild_tag_handle);
  DMSLmodel_map_single_handle_inventing_file_handle(model, nodetype_tag_handle);
  DMSLmodel_map_single_handle_inventing_file_handle(model, coord_tag_handle);
  DMSLmodel_map_single_handle_inventing_file_handle(model, size_tag_handle);
  DMSLmodel_map_single_handle_inventing_file_handle(model, bndbox_tag_handle);
  
  DMSLmodel_map_handles_inventing_file_handles(child_collection_id_1);
  DMSLmodel_map_handles_inventing_file_handles(child_collection_id_2);
  DMSLmodel_map_handles_inventing_file_handles(child_collection_id_3);
  DMSLmodel_map_handles_inventing_file_handles(child_collection_id_4);

  DMSLmodel_map_single_handle_inventing_file_handle(model, parent1_tag_handle);
  DMSLmodel_map_single_handle_inventing_file_handle(model, parent2_tag_handle);
  DMSLmodel_map_single_handle_inventing_file_handle(model, parent3_tag_handle);
  DMSLmodel_map_single_handle_inventing_file_handle(model, parent4_tag_handle);

/* Now we can tell damsel to execute everything. */
  damsel_request_t req;
  DMSLmodel_transfer_async(model, DAMSEL_TRANSFER_TYPE_WRITE, &req);
  damsel_status_t status;
  DMSLmodel_wait(req, &status);
  DMSLmodel_close(model);
  DMSLcontainer_close(block_collection_id);
  DMSLcontainer_close(block_connect_handle_collection_id);
  DMSLcontainer_close(block_connect_collection_id);
  DMSLcontainer_close(child_collection_id_1);
  DMSLcontainer_close(child_collection_id_2);
  DMSLcontainer_close(child_collection_id_3);
  DMSLcontainer_close(child_collection_id_4);
  DMSLlib_finalize(lib);

  MPI_Finalize();
  return 0;
}

