/* test create_sequence and writing to disk */

#include <assert.h>
#include <stdio.h>
#include <unistd.h>

#include "mpi.h"
#include "damsel.h"

#define FILENAME "output/test-overlap-bug.h5"

int main(int argc, char **argv) {
#ifndef DAMSEL_SINGLETHREAD
  int res;
#endif
  short int data[] = {3, 1, 4, 1, 5}; // some arbitrary data to write

  printf("%s\n", argv[0]);

#ifndef DAMSEL_SINGLETHREAD
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &res);
  assert(res == MPI_THREAD_MULTIPLE);
#else
  MPI_Init(&argc, &argv);
#endif

  damsel_library lib;
  damsel_err_t err = DMSLlib_init(&lib);

  damsel_model model;
  err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
  assert(model != NULL);

  /* define a tag which will store our data. the tag will be
     named 'somedata' */

  damsel_handle data_tag_handle = 0x10203;
  DMSLtag_define(model, &data_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "somedata");

  damsel_container vertex_container_id;
  err = DMSLcontainer_create_sequence(model,
    0x20000, 5, 0x1000, &vertex_container_id);


  /* now set the values of the somedata tag on the handles */
  DMSLmodel_map_tag(data, vertex_container_id, &data_tag_handle);

  /* at this point, the in-memory store knows about all the data that
     we have. next we will write the data out to disk.
     we need to invent some handles for the file store too. They can overlap
     with memstore handles, though here I make sure that they do not.
  */

  unlink(FILENAME); // maybe it doesn't exist, so we'll just silently ignore any error

  damsel_trait_bundle tb;
  err = DMSLtrait_create(&tb);
  DMSLtrait_put(tb, "mode", "WRITE");
  DMSLmodel_attach(model, FILENAME, MPI_COMM_WORLD, tb);
  DMSLtrait_release(tb);

  damsel_container vertex_container_id1;
  err = DMSLcontainer_create_sequence(model, 0x20000, 1, 0x1000, &vertex_container_id1);
  damsel_container vertex_container_id2;
  err = DMSLcontainer_create_sequence(model, 0x21000, 1, 0x1000, &vertex_container_id2);
  damsel_container vertex_container_id3;
  err = DMSLcontainer_create_sequence(model, 0x22000, 1, 0x1000, &vertex_container_id3);
  damsel_container vertex_container_id4;
  err = DMSLcontainer_create_sequence(model, 0x23000, 1, 0x1000, &vertex_container_id4);
  damsel_container vertex_container_id5;
  err = DMSLcontainer_create_sequence(model, 0x24000, 1, 0x1000, &vertex_container_id5);
  DMSLmodel_map_handles_inventing_file_handles(vertex_container_id1);
  DMSLmodel_map_handles_inventing_file_handles(vertex_container_id2);
  DMSLmodel_map_handles_inventing_file_handles(vertex_container_id3);
  DMSLmodel_map_handles_inventing_file_handles(vertex_container_id4);
  DMSLmodel_map_handles_inventing_file_handles(vertex_container_id5);

  /* we also need to map the handles for the tags which contain values
     that we want to store.
     This happens by creating a container containing the tag to map, and
     mapping that into the file store. */

  DMSLmodel_map_single_handle_inventing_file_handle(model, data_tag_handle);

  /* Now we can tell damsel to execute everything. */

  damsel_request_t req;
  DMSLmodel_transfer_async(model, DAMSEL_TRANSFER_TYPE_WRITE, &req);
  damsel_status_t status;
  DMSLmodel_wait(req, &status);


  DMSLmodel_close(model);
  model=NULL;

  DMSLcontainer_close(vertex_container_id);
  vertex_container_id=NULL;
  DMSLcontainer_close(vertex_container_id1);
  DMSLcontainer_close(vertex_container_id2);
  DMSLcontainer_close(vertex_container_id3);
  DMSLcontainer_close(vertex_container_id4);
  DMSLcontainer_close(vertex_container_id5);
  vertex_container_id1=NULL;
  vertex_container_id2=NULL;
  vertex_container_id3=NULL;
  vertex_container_id4=NULL;
  vertex_container_id5=NULL;

  DMSLlib_finalize(lib);
  lib=NULL;

  MPI_Finalize();
  return 0;
}

