/* test-pnetcdf-texattr: in test-pnetcdf, writing a text attribute seemed to
 * work but not reading it back.  Problem turned out to be missing calls in API
 * but this simple test made the error clear.  Plus, it provides a case of
 * 'BYTE' datatypes */


#include "mpi.h"
#include "damsel.h"
#include "damsel-internal.h"

#include <stdio.h>
#include <assert.h>

#include <string.h>

#include "damsel-sugar.h"

#define FILENAME "output/test-pnetcdf-textattr.dmsl"

damsel_handle handle_base=0x100;

#define REFERENCE_STRING "Hello World\n\0"
void write_text() {
    damsel_model model;
    damsel_trait_bundle tb;
    char data[] = REFERENCE_STRING;
    char attr_tag_name[] = "string";

    damsel_data attribute;

    damsel_err_t err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
    err = DMSLtrait_create(&tb);

    DMSLtrait_put(tb, "mode", "WRITE");
    
    DMSL_CHECK(DMSLmodel_attach(model, FILENAME, MPI_COMM_WORLD, tb));

    DMSLtrait_release(tb);

    attribute.tag_handle=(handle_base++);
    DMSL_CHECK(DMSLtag_define(model, &(attribute.tag_handle), 
		DAMSEL_DATA_TYPE_BYTES, attr_tag_name));

    err = DMSLcontainer_create_sequence(model,
	    handle_base, 13, 1, &attribute.container);
    handle_base += 13;

    DMSL_CHECK(DMSLmodel_map_tag(data, attribute.container, 
		&(attribute.tag_handle)));

    DMSL_CHECK(DMSLmodel_map_handles_inventing_file_handles(attribute.container));
    DMSL_CHECK(DMSLmodel_map_single_handle_inventing_file_handle(model, attribute.tag_handle));

    DMSLmodel_transfer_sync(model, DAMSEL_TRANSFER_TYPE_WRITE);

    DMSL_CHECK(DMSLmodel_close(model));
    DMSL_CHECK(DMSLcontainer_close(attribute.container));
}

void read_text()
{
    damsel_model model, fileside_model;
    damsel_model_internal *model_internal;
    damsel_trait_bundle tb;
    int len, i;
    char newdata[256];

    for(i=0; i<256; i++) newdata[i] = '!';


    damsel_data attr_mem, attr_file;

    damsel_err_t err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);

    err = DMSLtrait_create(&tb);
    DMSLtrait_put(tb, "mode", "READ");
    DMSL_CHECK(DMSLmodel_attach(model, FILENAME, MPI_COMM_WORLD, tb));
    DMSLtrait_release(tb);
    DMSL_CHECK(DMSLmodel_populate(model));

    model_internal = (damsel_model_internal *)model;
    fileside_model = (damsel_model) model_internal->attached_file;

    err = DMSLselect_tag_by_name(model, "string", &attr_file.tag_handle);
    err = DMSLcontainer_create_sequence(fileside_model,
	    attr_file.tag_handle, 1, 1, &attr_file.tag_container);

    attr_mem.tag_handle = handle_base++;
    err = DMSLcontainer_create_sequence(model,
	    attr_mem.tag_handle, 1, 1, &attr_mem.tag_container);

    DMSL_CHECK(DMSLmodel_map_handles(attr_mem.tag_container, attr_file.tag_container));

    attr_file.container = DMSLselect_handles_with_values(model, attr_file.tag_handle);
    len = DMSLcontainer_count(attr_file.container);
    assert(len == 13);
    
    attr_mem.container_id = handle_base++;
    err = DMSLcontainer_create_sequence(model, attr_mem.container_id,
	    13, 1, &attr_mem.container);

    DMSL_CHECK(DMSLmodel_map_handles(attr_mem.container, attr_file.container));

    DMSL_CHECK(DMSLmodel_map_tag(newdata, attr_mem.container, &(attr_mem.tag_handle)));

    DMSLmodel_transfer_sync(model, DAMSEL_TRANSFER_TYPE_READ);


    if (strcmp(REFERENCE_STRING, newdata)) {
	fprintf(stderr, "Expected %s got %s\n", REFERENCE_STRING, newdata);
	assert (1==0);
    }

    DMSL_CHECK(DMSLmodel_close(model));

    DMSL_CHECK(DMSLcontainer_close(attr_mem.container));
    DMSL_CHECK(DMSLcontainer_close(attr_mem.tag_container));
    DMSL_CHECK(DMSLcontainer_close(attr_file.container));
    DMSL_CHECK(DMSLcontainer_close(attr_file.tag_container));

}


int main(int argc, char ** argv) 
{
    int level;
#ifndef DAMSEL_SINGLETHREAD
    MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &level);
    assert(level == MPI_THREAD_MULTIPLE);
#else
    MPI_Init(&argc, &argv);
#endif
    damsel_library lib;
    damsel_err_t err = DMSLlib_init(&lib);

    write_text();
    read_text();

    DMSLlib_finalize(lib);
    MPI_Finalize();
    return 0;
}
