/* tests reading into a sequence */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>
#include <unistd.h>
#include <memory.h>

#include "damsel.h"
#include "damsel-internal.h"

int main(int argc, char **argv) {

  // we'll read the data into data[] and then check that
  // it matches up with target data check that
  // it matches up with target data
  short int targetdata[] = {3, 1, 4, 1, 5};
  short int data[5];

  printf("%s\n", argv[0]);
  if(argc!=2) {
    printf("specify filename of .h5 dump file to check\n");
    return 2;
  }

#ifndef DAMSEL_SINGLETHREAD
  int res;
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &res);
  assert(res == MPI_THREAD_MULTIPLE);
#else
  MPI_Init(&argc, &argv);
#endif

  damsel_library lib;
  damsel_err_t err = DMSLlib_init(&lib);

  damsel_model model;
  err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
  assert(model != NULL);
  damsel_model_internal *ms = (damsel_model_internal *)model;
  assert(ms != NULL);

  damsel_trait_bundle tb;
  err = DMSLtrait_create(&tb);
  DMSLtrait_put(tb, "mode", "READ");
  DMSLmodel_attach(model, argv[1], MPI_COMM_WORLD, tb);
  DMSLtrait_release(tb);
  DMSLmodel_populate(model);

  damsel_handle data_tag_handle = 0x31000;

  /* we'll create a sequence handle container with 5 handles in it. */

  damsel_container vertex_container_id;
  err = DMSLcontainer_create_sequence(model, 0x80900, 5, 0x5, &vertex_container_id);

  assert(vertex_container_id != NULL);

  damsel_container tag_container_id;
  err = DMSLcontainer_create_sequence(model, data_tag_handle, 1, 1, &tag_container_id);

  assert(tag_container_id != NULL);

  damsel_handle fileside_data_tag_handle;
  err = DMSLselect_tag_by_name(model, "somedata", &fileside_data_tag_handle);
  damsel_container file_tag_container_id;
  err = DMSLcontainer_create_sequence((damsel_model)ms->attached_file, fileside_data_tag_handle, 1, 1, &file_tag_container_id);
  DMSLmodel_map_handles(tag_container_id, file_tag_container_id);

  /* now set the values of the somedata tag on the handles */
  DMSLmodel_map_tag(data, vertex_container_id, &data_tag_handle);

  damsel_container filestore_vertex_container_id = DMSLselect_handles_with_values(model, fileside_data_tag_handle);

  int flength = DMSLcontainer_count(filestore_vertex_container_id);

  printf("expecting length 5, got length %d\n", flength);

  int cpos;
  for(cpos = 0; cpos<flength; cpos++) {
    printf("handle %d = %llx\n", cpos, DMSLcontainer_handle_at_position(filestore_vertex_container_id, cpos));
  }

  assert(flength == 5);

  // REGENERATE HERE
  unsigned id2size = DMSLcontainer_count(vertex_container_id);
  damsel_handle *id2space = malloc(sizeof(damsel_handle) * id2size);
  for(cpos=0; cpos < id2size; cpos++) {
    id2space[cpos] = DMSLcontainer_handle_at_position(vertex_container_id, cpos);
  }
  damsel_container vertex_container_id2;
  err = DMSLcontainer_create_vector(model, id2space, id2size, &vertex_container_id2);

  DMSLmodel_map_handles(vertex_container_id2, filestore_vertex_container_id);
  DMSLcontainer_release(filestore_vertex_container_id);

  damsel_request_t req;
  DMSLmodel_transfer_async(model, DAMSEL_TRANSFER_TYPE_READ, &req);
  damsel_status_t status;
  DMSLmodel_wait(req, &status);

  int ix;
  for(ix=0; ix<5; ix++) {
printf("Comparing read value %d with target value %d\n", data[ix], targetdata[ix]);
    if(targetdata[ix] != data[ix]) {
printf("fail!\n");
      return 1;
    }
  }

  DMSLmodel_close(model);

  DMSLcontainer_close(vertex_container_id);
  DMSLcontainer_close(tag_container_id);
  DMSLcontainer_close(file_tag_container_id);

  DMSLlib_finalize(lib);

  MPI_Finalize();
  return 0;
}

