/* test create_sequence and writing to disk */

#include <assert.h>
#include <stdio.h>
#include <unistd.h>

#include "mpi.h"
#include "damsel.h"

#define FILENAME "output/test-reopen-populate.h5"

void pass1();
void pass2();

short int data[] = {3, 1, 4, 1, 5}; // some arbitrary data to write

int main(int argc, char **argv) {
#ifndef DAMSEL_SINGLETHREAD
  int res;
#endif

  printf("%s\n", argv[0]);

#ifndef DAMSEL_SINGLETHREAD
  MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &res);
  assert(res == MPI_THREAD_MULTIPLE);
#else
  MPI_Init(&argc, &argv);
#endif

  damsel_library lib;
  damsel_err_t err = DMSLlib_init(&lib);

  pass1();
  pass2();


  DMSLlib_finalize(lib);
  lib=NULL;

  MPI_Finalize();
  return 0;
}

void pass1() {

  damsel_model model;
  damsel_err_t err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
  assert(model != NULL);

  /* define a tag which will store our data. the tag will be
     named 'somedata' */

  damsel_handle data_tag_handle = 0x10204;
  DMSLtag_define(model, &data_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "somedataFIRST");

  /* we'll create a sequence handle collection with 5 handles in it. */

  damsel_container vertex_collection_id;
  err = DMSLcontainer_create_sequence(model,
    0x20000, 5, 0x1000, &vertex_collection_id);

  /* now set the values of the somedata tag on the handles */
  DMSLmodel_map_tag(data, vertex_collection_id, &data_tag_handle);

  /* at this point, the in-memory store knows about all the data that
     we have. next we will write the data out to disk.
     we need to invent some handles for the file store too. They can overlap
     with memstore handles, though here I make sure that they do not.
  */

  unlink(FILENAME); // maybe it doesn't exist, so we'll just silently ignore any error

  damsel_trait_bundle tb;
  err = DMSLtrait_create(&tb);
  DMSLtrait_put(tb, "mode", "WRITE");
  DMSLmodel_attach(model, FILENAME, MPI_COMM_WORLD, tb);
  DMSLtrait_release(tb);

  DMSLmodel_map_handles_inventing_file_handles(vertex_collection_id); // REPLACE

  /* we also need to map the handles for the tags which contain values
     that we want to store.
     This happens by creating a collection containing the tag to map, and
     mapping that into the file store. */

  DMSLmodel_map_single_handle_inventing_file_handle(model, data_tag_handle);

  /* Now we can tell damsel to execute everything. */

  damsel_request_t req;
  DMSLmodel_transfer_async(model, DAMSEL_TRANSFER_TYPE_WRITE, &req);
  damsel_status_t status;
  DMSLmodel_wait(req, &status);


  DMSLmodel_close(model);
  model=NULL;

  DMSLcontainer_close(vertex_collection_id);
  vertex_collection_id=NULL;

}

void pass2() {
  damsel_model model;
  damsel_err_t err = DMSLmodel_create(DAMSEL_HANDLE_TYPE_HANDLE64, &model);
  assert(model != NULL);
  damsel_handle data_tag_handle = 0x10203;
  DMSLtag_define(model, &data_tag_handle, DAMSEL_DATA_TYPE_SHORTINT, "somedata");

  /* we'll create a sequence handle collection with 5 handles in it. */
  /* these handles will deliberately overlap the appside handles used in
     pass1() - because this is a new "session", so we get a new space. */

  /* we need to use the same tag handle, because the present implementation
     doesn't map tags between app side and file side properly (thats a
     separate bug */

  damsel_container vertex_collection_id;
  err = DMSLcontainer_create_sequence(model,
    0x20000, 5, 0x1000, &vertex_collection_id);
  DMSLmodel_map_tag(data, vertex_collection_id, &data_tag_handle);
  damsel_trait_bundle tb;
  err = DMSLtrait_create(&tb);
  DMSLtrait_put(tb, "mode", "REOPEN");
  DMSLmodel_attach(model, FILENAME, MPI_COMM_WORLD, tb);
  DMSLtrait_release(tb);

  DMSLmodel_populate(model);

  DMSLmodel_map_handles_inventing_file_handles(vertex_collection_id); // REPLACE
  DMSLmodel_map_single_handle_inventing_file_handle(model, data_tag_handle);
  damsel_request_t req;
  DMSLmodel_transfer_async(model, DAMSEL_TRANSFER_TYPE_WRITE, &req);
  damsel_status_t status;
  DMSLmodel_wait(req, &status);
  DMSLmodel_close(model);
  model=NULL;
  DMSLcontainer_close(vertex_collection_id);
  vertex_collection_id=NULL;
}
