/* test-subarray-read: first, blat out a big hunk of 3d data so we have
 * something to read. next, attempt to read a sub-region of that data. */

#include <stdio.h>
#include <unistd.h>

#include "damsel.h"
#include "damsel-internal.h"
#include "damsel-sugar.h"

#include <mpi.h>

#include <assert.h>

#define FILENAME "output/test-subarray-read.dmsl"
#define TAGNAME "array"



/* I thought there would be some clever way to initialize all the elements of
 * an arbitrary Nd array recursively. Might come back to the idea one day but
 * this is presently broken code. do not use */
void seed_buffer_recurse(int *buf, int ndims, int *dims, int seed, int *nitems)
{
    int i;

    if (ndims == 1) { /* base case */
	for (i=0; i<dims[0]; i++) {
	    buf[i] = seed + i;
	}
	*nitems += dims[0];
    } else {
	for(i=0; i<dims[0]; i++) {
	    fprintf(stderr, "seeding %dth element, %d items starting with %d\n",
		    *nitems, dims[1], seed + 10*(ndims-1)*i*dims[1]);
	    seed_buffer_recurse(buf+(*nitems), ndims-1, dims+1, 
		    seed + (10*(ndims-1))*i*dims[1], nitems);
	}
    }
}

void seed_buffer(int *buf, int ndims, int *dims)
{
    int i, j, k;
    /* i think there's a way to do this initialization recursively for any N
     * dimensions but I haven't worked it out yet */
    assert(ndims == 3);
    for (i=0; i< dims[0]; i++)
	for (j=0; j<dims[1]; j++)
	    for (k=0; k<dims[2]; k++)
		buf[i*dims[1]*dims[2] + j*dims[2] + k] = 
		    100*i*dims[1]*dims[2] + 10*j*dims[2] + k;
}

void seed_file(MPI_Comm comm, char *fname, int ndims, int *dims)
{
    damsel_model model;
    damsel_data stuffing;  /* happy thanksgiving */
    damsel_err_t err;
    int handle_counter = 900;
    int i;
    int64_t nitems;
    int *data;

    unlink(fname);
    model = DMSLmodel_attached_with_trait(comm, fname, "WRITE");

    nitems=1;
    for (i=0; i<ndims; i++) {
	if (dims[i] == 0) dims[i] = 1; /* what does a zero dimension mean? */
	nitems *= dims[i];
    }

    data = calloc(nitems, sizeof(int));
    seed_buffer(data, ndims, dims);

    err = DMSLcontainer_create_sequence(model, handle_counter++, 
	    nitems, 1, &stuffing.container);
    stuffing.tag_handle = handle_counter+=nitems;

    DMSL_CHECK(DMSLtag_define(model, &(stuffing.tag_handle), DAMSEL_DATA_TYPE_INTEGER,
	    TAGNAME));
    DMSL_CHECK(DMSLmodel_map_tag(data, stuffing.container, &(stuffing.tag_handle)));

    DMSL_CHECK(DMSLmodel_map_handles_inventing_file_handles(stuffing.container));
    DMSL_CHECK(DMSLmodel_map_single_handle_inventing_file_handle(model,
		stuffing.tag_handle));

    DMSLmodel_transfer_sync(model, DAMSEL_TRANSFER_TYPE_WRITE);

    DMSL_CHECK(DMSLcontainer_release(stuffing.container));
    DMSL_CHECK(DMSLmodel_close(model));

    free(data);

    return;
}

int subarray_read_1d_front(MPI_Comm comm, char *fname) {
    /* building up slowly: what if we want to read only a few items?  say the first 10?*/

    damsel_model model, fileside_model;
    damsel_model_internal *model_internal;
    damsel_data mem, file;
    damsel_container subset_container;
    damsel_err_t err;
    int *data, i, ndims;
    int64_t nitems, start, size;
    int nerrors=0, standard[] = {0,1,2,3,4,5,60,61,62,63};

    int handle_counter = 100;

    model = DMSLmodel_attached_with_trait(comm, fname, "READ");
    DMSL_CHECK(DMSLmodel_populate(model));

    model_internal= (damsel_model_internal*)model;
    fileside_model = (damsel_model) model_internal->attached_file;

    err = DMSLselect_tag_by_name(model, TAGNAME, &file.tag_handle);
    err = DMSLcontainer_create_sequence(fileside_model, 
	    file.tag_handle,  1, 1, &file.tag_container);

    mem.tag_handle = handle_counter ++;
    err = DMSLcontainer_create_sequence(model,
	    mem.tag_handle, 1, 1, &mem.tag_container);

    DMSL_CHECK(DMSLmodel_map_handles(mem.tag_container, file.tag_container));

    nitems=10;
    data = calloc(nitems, sizeof(int));

    ndims=1; size=120; start=0;
    subset_container = DMSLselect_subset_from_tag(model, 
	    file.tag_handle, ndims, &size, &start, &nitems);
  

    mem.container_id = handle_counter; handle_counter += nitems;
    err = DMSLcontainer_create_sequence(model, mem.container_id,
	    nitems, 1, &mem.container);

    DMSL_CHECK(DMSLmodel_map_handles(mem.container, subset_container));
    DMSL_CHECK(DMSLmodel_map_tag(data, mem.container, &(mem.tag_handle)));

    DMSLmodel_transfer_sync(model, DAMSEL_TRANSFER_TYPE_READ);

#ifdef VERBOSE
    file.container = DMSLselect_handles_with_values(model, file.tag_handle);
    int all = DMSLcontainer_count(file.container);
    fprintf(stderr, "read first %d of %d: ", nitems, all);
    for (i=0; i<nitems; i++) {
	fprintf(stderr, "||%d", data[i]);
    }
    fprintf(stderr, "||\n");
    DMSL_CHECK(DMSLcontainer_release(file.container));
#endif
    for (i=0; i<nitems; i++) {
	if (standard[i] != data[i])  nerrors++;
    }

    free(data);
    DMSL_CHECK(DMSLcontainer_release(mem.tag_container));
    DMSL_CHECK(DMSLcontainer_release(mem.container));
    DMSL_CHECK(DMSLcontainer_release(file.tag_container));
    DMSL_CHECK(DMSLcontainer_release(subset_container));
    DMSL_CHECK(DMSLmodel_close(model));
    return nerrors;
}
int subarray_read_1d_middle(MPI_Comm comm, char *fname) {
    /* a trivial change to subarray_read_1d_front: pick out some items from the
     * middle of the pile of handles  */

    damsel_model model, fileside_model;
    damsel_model_internal *model_internal;
    damsel_data mem, file;
    damsel_container subset_container;
    int *data, i, ndims; 
    int64_t nitems, size, start;
    int nerrors=0, standard[] = {180, 181, 182, 183, 184, 185, 240,241,242,243};
    damsel_err_t err;

    int handle_counter = 100;

    model = DMSLmodel_attached_with_trait(comm, fname, "READ");
    DMSL_CHECK(DMSLmodel_populate(model));

    model_internal= (damsel_model_internal*)model;
    fileside_model = (damsel_model) model_internal->attached_file;

    err = DMSLselect_tag_by_name(model, TAGNAME, &file.tag_handle);
    err = DMSLcontainer_create_sequence(fileside_model, 
	    file.tag_handle,  1, 1, &file.tag_container);

    mem.tag_handle = handle_counter ++;
    err = DMSLcontainer_create_sequence(model,
	    mem.tag_handle, 1, 1, &mem.tag_container);

    DMSL_CHECK(DMSLmodel_map_handles(mem.tag_container, file.tag_container));

    nitems=10;
    data = calloc(nitems, sizeof(int));
    
    ndims=1; size=120; start=6*3;
    subset_container = DMSLselect_subset_from_tag(model,
	    file.tag_handle, ndims, &size, &start, &nitems);
   

    mem.container_id = handle_counter; handle_counter += nitems;
    err = DMSLcontainer_create_sequence(model, mem.container_id,
	    nitems, 1, &mem.container);

    DMSL_CHECK(DMSLmodel_map_handles(mem.container, subset_container));
    DMSL_CHECK(DMSLmodel_map_tag(data, mem.container, &(mem.tag_handle)));

    DMSLmodel_transfer_sync(model, DAMSEL_TRANSFER_TYPE_READ);

#ifdef VERBOSE
    file.container = DMSLselect_handles_with_values(model, file.tag_handle);
    int all = DMSLcontainer_count(file.container);
    fprintf(stderr, "read %d of %d: ", nitems, all);
    for (i=0; i<nitems; i++) {
	fprintf(stderr, "||%d", data[i]);
    }
    fprintf(stderr, "||\n");
    DMSL_CHECK(DMSLcontainer_release(file.container));
#endif
    for (i=0; i<nitems; i++) {
	if (standard[i] != data[i])  nerrors++;
    }

    free(data);
    DMSL_CHECK(DMSLcontainer_release(mem.tag_container));
    DMSL_CHECK(DMSLcontainer_release(mem.container));
    DMSL_CHECK(DMSLcontainer_release(file.tag_container));
    DMSL_CHECK(DMSLcontainer_release(subset_container));
    DMSL_CHECK(DMSLmodel_close(model));
    return nerrors;
}
int subarray_read_2d(MPI_Comm comm, char *fname) {
    /* 2d subarray "just" a matter of selecting correct points */

    damsel_model model, fileside_model;
    damsel_model_internal *model_internal;
    damsel_data mem, file;
    damsel_container subset_container;
    damsel_err_t err;
    int nitems, *data, i;
    int ndims=2; 
    int64_t size[2], start[2], count[2];
    int nerrors=0, standard[] = {3001,3002,3003,3004,3005,
	3060,3061,3062,3063,3064,3065,3120,3121,3122,3123,3124,3125,3180,3181,3182,
	6001,6002,6003,6004,6005,
	6060,6061,6062,6063,6064,6065,6120,6121,6122,6123,6124,6125,6180,6181,6182};

    int handle_counter = 100;

    model = DMSLmodel_attached_with_trait(comm, fname, "READ");
    DMSL_CHECK(DMSLmodel_populate(model));

    model_internal= (damsel_model_internal*)model;
    fileside_model = (damsel_model) model_internal->attached_file;

    err = DMSLselect_tag_by_name(model, TAGNAME, &file.tag_handle);
    err = DMSLcontainer_create_sequence(fileside_model, 
	    file.tag_handle,  1, 1, &file.tag_container);

    mem.tag_handle = handle_counter ++;
    err = DMSLcontainer_create_sequence(model,
	    mem.tag_handle, 1, 1, &mem.tag_container);

    DMSL_CHECK(DMSLmodel_map_handles(mem.tag_container, file.tag_container));

    size[0] = 4; size[1] = 30; /* pretend 4x5x6 array is 2d */
    start[0] = 1; start[1] = 1;
    count[0] = 2; count[1] = 20;

    nitems = count[0] * count[1];
    data = calloc(nitems, sizeof(int));

    subset_container = DMSLselect_subset_from_tag(model, file.tag_handle, 
	    ndims, size, start, count);
   

    mem.container_id = handle_counter; handle_counter += nitems;
    err = DMSLcontainer_create_sequence(model, mem.container_id,
	    nitems, 1, &mem.container);

    DMSL_CHECK(DMSLmodel_map_handles(mem.container, subset_container));
    DMSL_CHECK(DMSLmodel_map_tag(data, mem.container, &(mem.tag_handle)));

    DMSLmodel_transfer_sync(model, DAMSEL_TRANSFER_TYPE_READ);

#ifdef VERBOSE
    file.container = DMSLselect_handles_with_values(model, file.tag_handle);
    int all = DMSLcontainer_count(file.container);
    fprintf(stderr, "read %d of %d: ", nitems, all);
    for (i=0; i<nitems; i++) {
	fprintf(stderr, "||%d", data[i]);
    }
    fprintf(stderr, "||\n");
    DMSL_CHECK(DMSLcontainer_release(file.container));
#endif
    for (i=0; i<nitems; i++) {
	if (standard[i] != data[i])  nerrors++;
    }

    free(data);
    DMSL_CHECK(DMSLcontainer_release(mem.tag_container));
    DMSL_CHECK(DMSLcontainer_release(mem.container));
    DMSL_CHECK(DMSLcontainer_release(file.tag_container));
    DMSL_CHECK(DMSLcontainer_release(subset_container));
    DMSL_CHECK(DMSLmodel_close(model));
    return nerrors;
}
int subarray_read_3d(MPI_Comm comm, char *filename) 
{
    /* 3d (and general Nd case) builds upon 2d */

    damsel_model model, fileside_model;
    damsel_model_internal *model_internal;
    damsel_data mem, file;
    damsel_container subset_container;
    damsel_err_t err;
    int nitems, *data, i;
    int ndims=3; 
    int64_t size[3], start[3], count[3];
    int nerrors=0, standard[] = {3061,3062,3121,3122,6061,6062,6121,6122};

    int handle_counter = 100;

    model = DMSLmodel_attached_with_trait(comm, filename, "READ");
    DMSL_CHECK(DMSLmodel_populate(model));

    model_internal= (damsel_model_internal*)model;
    fileside_model = (damsel_model) model_internal->attached_file;

    err = DMSLselect_tag_by_name(model, TAGNAME, &file.tag_handle);
    err = DMSLcontainer_create_sequence(fileside_model, 
	    file.tag_handle,  1, 1, &file.tag_container);

    mem.tag_handle = handle_counter ++;
    err = DMSLcontainer_create_sequence(model,
	    mem.tag_handle, 1, 1, &mem.tag_container);

    DMSL_CHECK(DMSLmodel_map_handles(mem.tag_container, file.tag_container));

    size[0] = 4; size[1] = 5; size[2] = 6; 
    start[0] = 1; start[1] = 1; start[2] = 1;
    count[0] = 2; count[1] = 2; count[2] = 2;

    nitems = DMSL_subprod(3, count);
    data = calloc(nitems, sizeof(int));

    subset_container = DMSLselect_subset_from_tag(model, file.tag_handle, 
	    ndims, size, start, count);
   

    mem.container_id = handle_counter; handle_counter += nitems;
    err = DMSLcontainer_create_sequence(model, mem.container_id,
	    nitems, 1, &mem.container);

    DMSL_CHECK(DMSLmodel_map_handles(mem.container, subset_container));
    DMSL_CHECK(DMSLmodel_map_tag(data, mem.container, &(mem.tag_handle)));

    DMSLmodel_transfer_sync(model, DAMSEL_TRANSFER_TYPE_READ);

#ifdef VERBOSE
    file.container = DMSLselect_handles_with_values(model, file.tag_handle);
    int all = DMSLcontainer_count(file.container);
    fprintf(stderr, "read %d of %d: ", nitems, all);
    for (i=0; i<nitems; i++) {
	fprintf(stderr, "||%d", data[i]);
    }
    fprintf(stderr, "||\n");
    DMSL_CHECK(DMSLcontainer_release(file.container));
#endif
    for (i=0; i<nitems; i++) {
	if (standard[i] != data[i])  nerrors++;
    }

    free(data);
    DMSL_CHECK(DMSLcontainer_release(mem.tag_container));
    DMSL_CHECK(DMSLcontainer_release(mem.container));
    DMSL_CHECK(DMSLcontainer_release(file.tag_container));
    DMSL_CHECK(DMSLcontainer_release(subset_container));
    DMSL_CHECK(DMSLmodel_close(model));
    return nerrors;
}
int main(int argc, char **argv) 
{
    int nprocs, rank;
#define NDIMS 3
    int dims[NDIMS] = {4, 5, 6};
    int nerrors;
    
#ifndef DAMSEL_SINGLETHREAD
    int threadlvl;
    MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &threadlvl);
    assert(threadlvl == MPI_THREAD_MULTIPLE);
#else
    MPI_Init(&argc, &argv);
#endif

    damsel_library lib;
    damsel_err_t err = DMSLlib_init(&lib);

    MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);

    seed_file(MPI_COMM_WORLD, FILENAME, 3, dims);

    nerrors = subarray_read_1d_front(MPI_COMM_WORLD, FILENAME);
    nerrors += subarray_read_1d_middle(MPI_COMM_WORLD, FILENAME);
    nerrors += subarray_read_2d(MPI_COMM_WORLD, FILENAME);
    nerrors += subarray_read_3d(MPI_COMM_WORLD, FILENAME);

    DMSLlib_finalize(lib);
    MPI_Finalize();
    if (nerrors == 0)
	return 0;
    else
	return -nerrors;
}
